/***************************************************************************
 *   Copyright (C) 2008 by Jakub Judas   *
 *   cleb@cleb-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "trayplay.h"

int main(int argc, char *argv[])
{
      QApplication app(argc, argv);
      int i;
      QString cmdline;
      if(argc>1) cmdline=QString::fromLocal8Bit(argv[1]);
      if(argc>2){ //spaces in filename
	      for(i=2;i<argc;i++){
		      cmdline+=" ";
		      cmdline+=QString::fromLocal8Bit(argv[i]);
	      }
      }
      QDBusInterface remoteApp( "cz.nodomain.trayplay.TrayPlay", "/Tray",
				"" );
      QDBusReply<QString> reply = remoteApp.call( "newInstance",cmdline);
      
      if(reply.isValid()) return 0;
      
      QIcon *icon=new QIcon("/usr/lib/trayplay/tp.png");
      Tray *systray;
      if(argc<2) systray=new Tray(icon);
      else systray=new Tray(icon,cmdline);
      systray->show();
      TrayAdaptor *adaptor = new TrayAdaptor(systray);
      QDBusConnection bus = QDBusConnection::sessionBus(); 
      bus.interface()->registerService("cz.nodomain.trayplay.TrayPlay");
      bus.registerObject("/Tray", adaptor,
			 QDBusConnection::ExportAllSlots);
      QApplication::setQuitOnLastWindowClosed(false);
      app.exec();
      delete systray;
      return 0;
}
Tray::Tray(QIcon *icon,QString filename):QSystemTrayIcon(*icon)
{
	playIcon=icon;
	pauseIcon=new QIcon("/usr/lib/trayplay/tpa.png");
	stopIcon=new QIcon("/usr/lib/trayplay/ts.png");
	connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(Clicked(QSystemTrayIcon::ActivationReason)));
	media = new Phonon::MediaObject();
	Phonon::AudioOutput *audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory);
	Phonon::createPath(media, audioOutput);
	
	openAction = new QAction(tr("&Open file"), this);
	playAction = new QAction(tr("&Play/Pause"), this);
	stopAction = new QAction(tr("&Stop"), this);
	quitAction = new QAction(tr("&Quit"), this);
	nextAction = new QAction(tr("&Next"), this);
	connect(openAction, SIGNAL(triggered()), this, SLOT(FileOpen()));
	connect(playAction, SIGNAL(triggered()), this, SLOT(Play()));
	connect(stopAction, SIGNAL(triggered()), this, SLOT(Stop()));
	connect(quitAction, SIGNAL(triggered()), this, SLOT(Quit()));
	connect(nextAction, SIGNAL(triggered()), this, SLOT(Next()));
	
	menu=new QMenu();
	menu->addAction(openAction);
	menu->addAction(playAction);
	menu->addAction(stopAction);
	menu->addAction(nextAction);
	menu->addAction(quitAction);
	setContextMenu(menu);
	source=NULL;
	if(!filename.isEmpty()){
		playlist.addFile(filename);
		source=playlist.getNext();
		startPlaying();
	}
	connect(media, SIGNAL(finished()), this, SLOT(Next()));
	
};
Tray::~Tray()
{
	delete media;
	delete source;
	delete menu;
	delete playIcon;
	delete pauseIcon;
	delete stopIcon;	
	delete openAction;
	delete playAction;
	delete stopAction;
	delete quitAction;
	delete nextAction;
}
void Tray::startPlaying()
{
	media->setCurrentSource(*source);
	media->play();	
	setToolTip("Systray player 0.9.1 now playing "+playlist.getActiveFileName());
}
void Tray::FileOpen()
{
	fileName = QFileDialog::getOpenFileName(0,tr("Open File"), playlist.getLastFileDir(), tr("Audio files (*.mp3 *.flac *.wma)"));
	if (!fileName.isEmpty())
	{
		playlist.addFile(fileName);
		if(media->state()!=Phonon::PlayingState){
			source=playlist.getNext();
			startPlaying();
		}
	}
}
void Tray::Stop()
{
	media->stop();
	setIcon(*stopIcon);
}
void Tray::Play()
{
	if(media->state()==Phonon::PlayingState){
		 media->pause();
		 setIcon(*pauseIcon);
	}
	else 
	{
		media->play();
		setIcon(*playIcon);
	}
}
void Tray::Quit()
{
	QApplication::quit();
}
void Tray::Next()
{
	media->stop();
	delete source;
	source=playlist.getNext();
	if(source){
		startPlaying();
	}
}
void Tray::Clicked(QSystemTrayIcon::ActivationReason reason)
{
	printf("Clicked");
	if(reason==QSystemTrayIcon::Trigger){
		Play();
	}
}
void Tray::newInstance(QString arg){
	playlist.addFile(arg);
	if(media->state()!=Phonon::PlayingState&&media->state()!=Phonon::PausedState) Next();
	
}
Playlist::Playlist()
{
	active=0;
}
void Playlist::addFile(QString filename)
{
	playlist.push_back(filename);
}
int Playlist::setActive(int id)
{
	active=id;
	return 0;
}
QString Playlist::getActiveFileName()
{
	return playlist[active-1];
}
QString Playlist::getLastFileDir()
{
	if(playlist.size()) return playlist.back();
	else return "/home";
}
Phonon::MediaSource *Playlist::getNext()
{
	if(active<playlist.size()){
		active++;
		return new Phonon::MediaSource(playlist[active-1]);
	}
	else return NULL;	
}
TrayAdaptor::TrayAdaptor(Tray *t):QDBusAbstractAdaptor(t)
{
	tray=t;	
}
QString TrayAdaptor::newInstance(QString arg)
{
	tray->newInstance(arg);
	return QString("OK");
}
